'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.global = undefined;

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

function _interopRequireDefault(obj) {
  return obj && obj.__esModule ? obj : { default: obj };
}

/*
 * Copyright (c) 2015-present, Parse, LLC.
 * All rights reserved.
 *
 * This source code is licensed under the BSD-style license found in the
 * LICENSE file in the root directory of this source tree. An additional grant
 * of patent rights can be found in the PATENTS file in the same directory.
 *
 * 
 */

function requireMethods(name, methods, controller) {
  methods.forEach(function (func) {
    if (typeof controller[func] !== 'function') {
      throw new Error(name + ' must implement ' + func + '()');
    }
  });
}

var CoreManagerType = function () {
  function CoreManagerType() {
    (0, _classCallCheck3.default)(this, CoreManagerType);

    this.config = {
      // Defaults
      IS_NODE: typeof process !== 'undefined' && !!process.versions && !!process.versions.node && !process.versions.electron,
      REQUEST_ATTEMPT_LIMIT: 5,
      SERVER_URL: 'https://api.parse.com/1',
      LIVEQUERY_SERVER_URL: null,
      VERSION: 'js' + '2.0.3',
      APPLICATION_ID: null,
      JAVASCRIPT_KEY: null,
      MASTER_KEY: null,
      USE_MASTER_KEY: false,
      PERFORM_USER_REWRITE: true,
      FORCE_REVOCABLE_SESSION: false
    };
  }

  (0, _createClass3.default)(CoreManagerType, [{
    key: 'get',
    value: function (key) {
      if (this.config.hasOwnProperty(key)) {
        return this.config[key];
      }
      throw new Error('Configuration key not found: ' + key);
    }
  }, {
    key: 'set',
    value: function (key, value) {
      this.config[key] = value;
    }

    /* Specialized Controller Setters/Getters */

  }, {
    key: 'setAnalyticsController',
    value: function (controller) {
      requireMethods('AnalyticsController', ['track'], controller);
      this.config['AnalyticsController'] = controller;
    }
  }, {
    key: 'getAnalyticsController',
    value: function () {
      return this.config['AnalyticsController'];
    }
  }, {
    key: 'setCloudController',
    value: function (controller) {
      requireMethods('CloudController', ['run'], controller);
      this.config['CloudController'] = controller;
    }
  }, {
    key: 'getCloudController',
    value: function () {
      return this.config['CloudController'];
    }
  }, {
    key: 'setConfigController',
    value: function (controller) {
      requireMethods('ConfigController', ['current', 'get'], controller);
      this.config['ConfigController'] = controller;
    }
  }, {
    key: 'getConfigController',
    value: function () {
      return this.config['ConfigController'];
    }
  }, {
    key: 'setFileController',
    value: function (controller) {
      requireMethods('FileController', ['saveFile', 'saveBase64'], controller);
      this.config['FileController'] = controller;
    }
  }, {
    key: 'getFileController',
    value: function () {
      return this.config['FileController'];
    }
  }, {
    key: 'setInstallationController',
    value: function (controller) {
      requireMethods('InstallationController', ['currentInstallationId'], controller);
      this.config['InstallationController'] = controller;
    }
  }, {
    key: 'getInstallationController',
    value: function () {
      return this.config['InstallationController'];
    }
  }, {
    key: 'setObjectController',
    value: function (controller) {
      requireMethods('ObjectController', ['save', 'fetch', 'destroy'], controller);
      this.config['ObjectController'] = controller;
    }
  }, {
    key: 'getObjectController',
    value: function () {
      return this.config['ObjectController'];
    }
  }, {
    key: 'setObjectStateController',
    value: function (controller) {
      requireMethods('ObjectStateController', ['getState', 'initializeState', 'removeState', 'getServerData', 'setServerData', 'getPendingOps', 'setPendingOp', 'pushPendingState', 'popPendingState', 'mergeFirstPendingState', 'getObjectCache', 'estimateAttribute', 'estimateAttributes', 'commitServerChanges', 'enqueueTask', 'clearAllState'], controller);

      this.config['ObjectStateController'] = controller;
    }
  }, {
    key: 'getObjectStateController',
    value: function () {
      return this.config['ObjectStateController'];
    }
  }, {
    key: 'setPushController',
    value: function (controller) {
      requireMethods('PushController', ['send'], controller);
      this.config['PushController'] = controller;
    }
  }, {
    key: 'getPushController',
    value: function () {
      return this.config['PushController'];
    }
  }, {
    key: 'setQueryController',
    value: function (controller) {
      requireMethods('QueryController', ['find', 'aggregate'], controller);
      this.config['QueryController'] = controller;
    }
  }, {
    key: 'getQueryController',
    value: function () {
      return this.config['QueryController'];
    }
  }, {
    key: 'setRESTController',
    value: function (controller) {
      requireMethods('RESTController', ['request', 'ajax'], controller);
      this.config['RESTController'] = controller;
    }
  }, {
    key: 'getRESTController',
    value: function () {
      return this.config['RESTController'];
    }
  }, {
    key: 'setSchemaController',
    value: function (controller) {
      requireMethods('SchemaController', ['get', 'create', 'update', 'delete', 'send', 'purge'], controller);
      this.config['SchemaController'] = controller;
    }
  }, {
    key: 'getSchemaController',
    value: function () {
      return this.config['SchemaController'];
    }
  }, {
    key: 'setSessionController',
    value: function (controller) {
      requireMethods('SessionController', ['getSession'], controller);
      this.config['SessionController'] = controller;
    }
  }, {
    key: 'getSessionController',
    value: function () {
      return this.config['SessionController'];
    }
  }, {
    key: 'setStorageController',
    value: function (controller) {
      if (controller.async) {
        requireMethods('An async StorageController', ['getItemAsync', 'setItemAsync', 'removeItemAsync'], controller);
      } else {
        requireMethods('A synchronous StorageController', ['getItem', 'setItem', 'removeItem'], controller);
      }
      this.config['StorageController'] = controller;
    }
  }, {
    key: 'getStorageController',
    value: function () {
      return this.config['StorageController'];
    }
  }, {
    key: 'setAsyncStorage',
    value: function (storage) {
      this.config['AsyncStorage'] = storage;
    }
  }, {
    key: 'getAsyncStorage',
    value: function () {
      return this.config['AsyncStorage'];
    }
  }, {
    key: 'setUserController',
    value: function (controller) {
      requireMethods('UserController', ['setCurrentUser', 'currentUser', 'currentUserAsync', 'signUp', 'logIn', 'become', 'logOut', 'requestPasswordReset', 'upgradeToRevocableSession', 'linkWith'], controller);
      this.config['UserController'] = controller;
    }
  }, {
    key: 'getUserController',
    value: function () {
      return this.config['UserController'];
    }
  }, {
    key: 'setLiveQueryController',
    value: function (controller) {
      requireMethods('LiveQueryController', ['subscribe', 'unsubscribe', 'open', 'close'], controller);
      this.config['LiveQueryController'] = controller;
    }
  }, {
    key: 'getLiveQueryController',
    value: function () {
      return this.config['LiveQueryController'];
    }
  }, {
    key: 'setHooksController',
    value: function (controller) {
      requireMethods('HooksController', ['create', 'get', 'update', 'remove'], controller);
      this.config['HooksController'] = controller;
    }
  }, {
    key: 'getHooksController',
    value: function () {
      return this.config['HooksController'];
    }
  }, {
    key: 'setTasksController',
    value: function (controller) {
      requireMethods('TasksController', ['create', 'get', 'update', 'remove'], controller);
      this.config['TasksController'] = controller;
    }
  }, {
    key: 'getTasksController',
    value: function () {
      return this.config['TasksController'];
    }
  }]);
  return CoreManagerType;
}();

exports.default = CoreManagerType;
var global = exports.global = new CoreManagerType();