'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

exports.default = function (CoreManager, Exports) {

  var ParsePromise = (0, _include2.default)('./ParsePromise', CoreManager);

  var XHR = null;
  if (typeof XMLHttpRequest !== 'undefined') {
    XHR = XMLHttpRequest;
  }


  function getDatonisUrl() {
    var datonisUrl = CoreManager.get('DATONIS_URL');
    if (datonisUrl[datonisUrl.length - 1] !== '/') {
      datonisUrl += '/';
    }
    return datonisUrl;
  }

  var Datonis = function () {

    /**
     * @param {(String)} datonisToken Datonis auth token
     * @param {(Number)} version Datonis api version
     */

    /**
     * @property datonisToken
     * @type String
     */
    function Datonis(datonisToken, version) {
      (0, _classCallCheck3.default)(this, Datonis);

      if (typeof datonisToken === 'string') {
        this.datonisToken = datonisToken;
      } else {
        throw new TypeError('Datonis token must be of type string.');
      }
      version = version || 3;
      if (typeof version === 'number') {
        this.version = version;
      } else {
        throw new TypeError('Datonis version must be of type number.');
      }
    }

    /**
     * Get things.
     * @param {Hash} data
     * @param {Hash} options
     * @return {Parse.Promise} A promise that is fulfilled when the ajax completes.
     */

    /**
     * @property version
     * @type Number
     */

    (0, _createClass3.default)(Datonis, [{
      key: 'getThings',
      value: function () {
        var data = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
        var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

        var url = getDatonisUrl() + 'api/v' + this.version + '/things';

        return this.ajax('GET', url, data, options);
      }

      /**
       * Get things.
       * @param {Hash} data
       * @param {Hash} options
       * @return {Parse.Promise} A promise that is fulfilled when the ajax completes.
       */

    }, {
      key: 'getThingRawEvents',
      value: function () {
        var data = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
        var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

        var url = getDatonisUrl() + 'api/v' + this.version + '/datonis_query/thing_data';

        return this.ajax('POST', url, data, options);
      }

      /**
       * Get things.
       * @param {Hash} data
       * @param {Hash} options
       * @return {Parse.Promise} A promise that is fulfilled when the ajax completes.
       */

    }, {
      key: 'getThingWaypoints',
      value: function (thing_key) {
        var data = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
        var options = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

        var url = getDatonisUrl() + 'api/v' + this.version + '/things/' + thing_key + '/waypoints';

        return this.ajax('GET', url, data, options);
      }

      /**
       * Get thing aggregated data. Function defination can be improved in future in case any additional computation required
       * from sdk.
       * @param {Hash} data
       * @param {Hash} options
       * @return {Parse.Promise} A promise that is fulfilled when the ajax completes.
       */

    }, {
      key: 'getThingAggregateData',
      value: function () {
        var data = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
        var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

        var url = getDatonisUrl() + 'api/v' + this.version + '/datonis_query/thing_aggregated_data';

        return this.ajax('POST', url, data, options);
      }
    }, {
      key: 'ajax',
      value: function (method, url, data, headers) {
        var _this = this;
        var promise = new ParsePromise.default();

        (function () {
          if (XHR == null) {
            throw new Error('Cannot make a request: No definition of XMLHttpRequest was found.');
          }
          var handled = false;
          var xhr = new XHR();

          xhr.onreadystatechange = function () {
            if (xhr.readyState !== 4 || handled) {
              return;
            }
            handled = true;

            if (xhr.status >= 200 && xhr.status < 300) {
              var response;
              try {
                response = JSON.parse(xhr.responseText);
              } catch (e) {
                promise.reject(e.toString());
              }
              if (response) {
                promise.resolve(response, xhr.status, xhr);
              }
            } else {
              promise.reject(xhr);
            }
          };

          headers = headers || {};
          headers['X-Auth-Token'] = _this.datonisToken;
          if (typeof headers['Content-Type'] !== 'string') {
            headers['Content-Type'] = 'application/json'; // Avoid pre-flight
          }

          xhr.open(method, url, true);
          for (var h in headers) {
            xhr.setRequestHeader(h, headers[h]);
          }
          xhr.send(data);
        })();

        return promise;
      }
    }]);
    return Datonis;
  }();

  Exports.default = Datonis;
};

var _include = require('./include');

var _include2 = _interopRequireDefault(_include);

function _interopRequireDefault(obj) {
  return obj && obj.__esModule ? obj : { default: obj };
}