'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

var _typeof2 = require('babel-runtime/helpers/typeof');

var _typeof3 = _interopRequireDefault(_typeof2);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

exports.default = function (CoreManager, Exports) {

  var decode = (0, _include2.default)('./decode', CoreManager);
  (0, _include2.default)('./encode', CoreManager);

  var _escape = (0, _include2.default)('./escape', CoreManager);
  var ParseError = (0, _include2.default)('./ParseError', CoreManager);
  var ParsePromise = (0, _include2.default)('./ParsePromise', CoreManager);
  var Storage = (0, _include2.default)('./Storage', CoreManager);

  /**
   * Parse.Config is a local representation of configuration data that
   * can be set from the Parse dashboard.
   *
   * @alias Parse.Config
   */

  var ParseConfig = function () {
    function ParseConfig() {
      (0, _classCallCheck3.default)(this, ParseConfig);

      this.attributes = {};
      this._escapedAttributes = {};
    }

    /**
     * Gets the value of an attribute.
     * @param {String} attr The name of an attribute.
     */

    (0, _createClass3.default)(ParseConfig, [{
      key: 'get',
      value: function (attr) {
        return this.attributes[attr];
      }

      /**
       * Gets the HTML-escaped value of an attribute.
       * @param {String} attr The name of an attribute.
       */

    }, {
      key: 'escape',
      value: function (attr) {
        var html = this._escapedAttributes[attr];
        if (html) {
          return html;
        }
        var val = this.attributes[attr];
        var escaped = '';
        if (val != null) {
          escaped = _escape.default(val.toString());
        }
        this._escapedAttributes[attr] = escaped;
        return escaped;
      }

      /**
       * Retrieves the most recently-fetched configuration object, either from
       * memory or from local storage if necessary.
       *
       * @static
       * @return {Config} The most recently-fetched Parse.Config if it
       *     exists, else an empty Parse.Config.
       */

    }], [{
      key: 'current',
      value: function () {
        var controller = CoreManager.getConfigController();
        return controller.current();
      }

      /**
       * Gets a new configuration object from the server.
       * @static
       * @param {Object} options A Backbone-style options object.
       * Valid options are:<ul>
       *   <li>success: Function to call when the get completes successfully.
       *   <li>error: Function to call when the get fails.
       * </ul>
       * @return {Parse.Promise} A promise that is resolved with a newly-created
       *     configuration object when the get completes.
       */

    }, {
      key: 'get',
      value: function (options) {
        options = options || {};

        var controller = CoreManager.getConfigController();
        return controller.get()._thenRunCallbacks(options);
      }
    }]);
    return ParseConfig;
  }();

  Exports.default = ParseConfig;

  var currentConfig = null;

  var CURRENT_CONFIG_KEY = 'currentConfig';

  function decodePayload(data) {
    try {
      var json = JSON.parse(data);
      if (json && (typeof json === 'undefined' ? 'undefined' : (0, _typeof3.default)(json)) === 'object') {
        return decode.default(json);
      }
    } catch (e) {
      return null;
    }
  }

  CoreManager.setConfigController({
    current: function () {
      if (currentConfig) {
        return currentConfig;
      }

      var config = new ParseConfig();
      var storagePath = Storage.generatePath(CURRENT_CONFIG_KEY);
      var configData;
      if (!Storage.async()) {
        configData = Storage.getItem(storagePath);

        if (configData) {
          var attributes = decodePayload(configData);
          if (attributes) {
            config.attributes = attributes;
            currentConfig = config;
          }
        }
        return config;
      }
      // Return a promise for async storage controllers
      return Storage.getItemAsync(storagePath).then(function (configData) {
        if (configData) {
          var attributes = decodePayload(configData);
          if (attributes) {
            config.attributes = attributes;
            currentConfig = config;
          }
        }
        return config;
      });
    },
    get: function () {
      var RESTController = CoreManager.getRESTController();

      return RESTController.request('GET', 'config', {}, {}).then(function (response) {
        if (!response || !response.params) {
          var error = new ParseError.default(ParseError.default.INVALID_JSON, 'Config JSON response invalid.');
          return ParsePromise.default.error(error);
        }

        var config = new ParseConfig();
        config.attributes = {};
        for (var attr in response.params) {
          config.attributes[attr] = decode.default(response.params[attr]);
        }
        currentConfig = config;
        return Storage.setItemAsync(Storage.generatePath(CURRENT_CONFIG_KEY), (0, _stringify2.default)(response.params)).then(function () {
          return config;
        });
      });
    }
  });
};

var _include = require('./include');

var _include2 = _interopRequireDefault(_include);

function _interopRequireDefault(obj) {
  return obj && obj.__esModule ? obj : { default: obj };
}