'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

exports.default = function (CoreManager, Exports) {

  var ParseOp = (0, _include2.default)('./ParseOp', CoreManager);
  (0, _include2.default)('./ParseObject', CoreManager);

  var ParseQuery = (0, _include2.default)('./ParseQuery', CoreManager);

  /**
   * Creates a new Relation for the given parent object and key. This
   * constructor should rarely be used directly, but rather created by
   * Parse.Object.relation.
   *
   * <p>
   * A class that is used to access all of the children of a many-to-many
   * relationship.  Each instance of Parse.Relation is associated with a
   * particular parent object and key.
   * </p>
   * @alias Parse.Relation
   */

  var ParseRelation = function () {

    /**
     * @param {Parse.Object} parent The parent of this relation.
     * @param {String} key The key for this relation on the parent.
     */
    function ParseRelation(parent, key) {
      (0, _classCallCheck3.default)(this, ParseRelation);

      this.parent = parent;
      this.key = key;
      this.targetClassName = null;
    }

    /*
     * Makes sure that this relation has the right parent and key.
     */

    (0, _createClass3.default)(ParseRelation, [{
      key: '_ensureParentAndKey',
      value: function (parent, key) {
        this.key = this.key || key;
        if (this.key !== key) {
          throw new Error('Internal Error. Relation retrieved from two different keys.');
        }
        if (this.parent) {
          if (this.parent.className !== parent.className) {
            throw new Error('Internal Error. Relation retrieved from two different Objects.');
          }
          if (this.parent.id) {
            if (this.parent.id !== parent.id) {
              throw new Error('Internal Error. Relation retrieved from two different Objects.');
            }
          } else if (parent.id) {
            this.parent = parent;
          }
        } else {
          this.parent = parent;
        }
      }

      /**
       * Adds a Parse.Object or an array of Parse.Objects to the relation.
          * @param {} objects The item or items to add.
       */

    }, {
      key: 'add',
      value: function (objects) {
        if (!Array.isArray(objects)) {
          objects = [objects];
        }

        var change = new ParseOp.RelationOp(objects, []);
        var parent = this.parent;
        if (!parent) {
          throw new Error('Cannot add to a Relation without a parent');
        }
        parent.set(this.key, change);
        this.targetClassName = change._targetClassName;
        return parent;
      }

      /**
       * Removes a Parse.Object or an array of Parse.Objects from this relation.
          * @param {} objects The item or items to remove.
       */

    }, {
      key: 'remove',
      value: function (objects) {
        if (!Array.isArray(objects)) {
          objects = [objects];
        }

        var change = new ParseOp.RelationOp([], objects);
        if (!this.parent) {
          throw new Error('Cannot remove from a Relation without a parent');
        }
        this.parent.set(this.key, change);
        this.targetClassName = change._targetClassName;
      }

      /**
       * Returns a JSON version of the object suitable for saving to disk.
          * @return {Object}
       */

    }, {
      key: 'toJSON',
      value: function () {
        return {
          __type: 'Relation',
          className: this.targetClassName
        };
      }

      /**
       * Returns a Parse.Query that is limited to objects in this
       * relation.
          * @return {Parse.Query}
       */

    }, {
      key: 'query',
      value: function () {
        var query;
        var parent = this.parent;
        if (!parent) {
          throw new Error('Cannot construct a query for a Relation without a parent');
        }
        if (!this.targetClassName) {
          query = new ParseQuery.default(parent.className);
          query._extraOptions.redirectClassNameForKey = this.key;
        } else {
          query = new ParseQuery.default(this.targetClassName);
        }
        query._addCondition('$relatedTo', 'object', {
          __type: 'Pointer',
          className: parent.className,
          objectId: parent.id
        });
        query._addCondition('$relatedTo', 'key', this.key);

        return query;
      }
    }]);
    return ParseRelation;
  }();

  Exports.default = ParseRelation;
};

var _include = require('./include');

var _include2 = _interopRequireDefault(_include);

function _interopRequireDefault(obj) {
  return obj && obj.__esModule ? obj : { default: obj };
}