'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof2 = require('babel-runtime/helpers/typeof');

var _typeof3 = _interopRequireDefault(_typeof2);

var _getPrototypeOf = require('babel-runtime/core-js/object/get-prototype-of');

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

exports.default = function (CoreManager, Exports) {

  var isRevocableSession = (0, _include2.default)('./isRevocableSession', CoreManager);
  var ParseObject = (0, _include2.default)('./ParseObject', CoreManager);
  var ParsePromise = (0, _include2.default)('./ParsePromise', CoreManager);
  var ParseUser = (0, _include2.default)('./ParseUser', CoreManager);

  /**
   * <p>A Parse.Session object is a local representation of a revocable session.
   * This class is a subclass of a Parse.Object, and retains the same
   * functionality of a Parse.Object.</p>
   * @alias Parse.Session
   * @extends Parse.Object
   */

  var ParseSession = function (_ParseObject$default) {
    (0, _inherits3.default)(ParseSession, _ParseObject$default);

    /**
     * 
     * @param {Object} attributes The initial set of data to store in the user.
     */
    function ParseSession(attributes) {
      (0, _classCallCheck3.default)(this, ParseSession);

      var _this = (0, _possibleConstructorReturn3.default)(this, (ParseSession.__proto__ || (0, _getPrototypeOf2.default)(ParseSession)).call(this, '_Session'));

      if (attributes && (typeof attributes === 'undefined' ? 'undefined' : (0, _typeof3.default)(attributes)) === 'object') {
        if (!_this.set(attributes || {})) {
          throw new Error('Can\'t create an invalid Session');
        }
      }
      return _this;
    }

    /**
     * Returns the session token string.
        * @return {String}
     */

    (0, _createClass3.default)(ParseSession, [{
      key: 'getSessionToken',
      value: function () {
        var token = this.get('sessionToken');
        if (typeof token === 'string') {
          return token;
        }
        return '';
      }
    }], [{
      key: 'readOnlyAttributes',
      value: function () {
        return ['createdWith', 'expiresAt', 'installationId', 'restricted', 'sessionToken', 'user'];
      }

      /**
       * Retrieves the Session object for the currently logged in session.
          * @static
       * @return {Parse.Promise} A promise that is resolved with the Parse.Session
       *   object after it has been fetched. If there is no current user, the
       *   promise will be rejected.
       */

    }, {
      key: 'current',
      value: function (options) {
        options = options || {};
        var controller = CoreManager.getSessionController();

        var sessionOptions = {};
        if (options.hasOwnProperty('useMasterKey')) {
          sessionOptions.useMasterKey = options.useMasterKey;
        }
        return ParseUser.default.currentAsync().then(function (user) {
          if (!user) {
            return ParsePromise.default.error('There is no current user.');
          }
          user.getSessionToken();

          sessionOptions.sessionToken = user.getSessionToken();
          return controller.getSession(sessionOptions);
        });
      }

      /**
       * Determines whether the current session token is revocable.
       * This method is useful for migrating Express.js or Node.js web apps to
       * use revocable sessions. If you are migrating an app that uses the Parse
       * SDK in the browser only, please use Parse.User.enableRevocableSession()
       * instead, so that sessions can be automatically upgraded.
          * @static
       * @return {Boolean}
       */

    }, {
      key: 'isCurrentSessionRevocable',
      value: function () {
        var currentUser = ParseUser.default.current();
        if (currentUser) {
          return isRevocableSession.default(currentUser.getSessionToken() || '');
        }
        return false;
      }
    }]);
    return ParseSession;
  }(ParseObject.default);

  Exports.default = ParseSession;

  ParseObject.default.registerSubclass('_Session', ParseSession);

  CoreManager.setSessionController({
    getSession: function (options) {
      var RESTController = CoreManager.getRESTController();
      var session = new ParseSession();

      return RESTController.request('GET', 'sessions/me', {}, options).then(function (sessionData) {
        session._finishFetch(sessionData);
        session._setExisted(true);
        return session;
      });
    }
  });
};

var _include = require('./include');

var _include2 = _interopRequireDefault(_include);

function _interopRequireDefault(obj) {
  return obj && obj.__esModule ? obj : { default: obj };
}