'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _assign = require('babel-runtime/core-js/object/assign');

var _assign2 = _interopRequireDefault(_assign);

exports.default = function (CoreManager, Exports) {

  var ParsePromise = (0, _include2.default)('./ParsePromise', CoreManager);

  CoreManager.setStorageController((0, _include2.default)('./StorageController.browser', CoreManager));


  (0, _assign2.default)(Exports, {
    async: function () {
      var controller = CoreManager.getStorageController();
      return !!controller.async;
    },
    getItem: function (path) {
      var controller = CoreManager.getStorageController();
      if (controller.async === 1) {
        throw new Error('Synchronous storage is not supported by the current storage controller');
      }
      return controller.getItem(path);
    },
    getItemAsync: function (path) {
      var controller = CoreManager.getStorageController();
      if (controller.async === 1) {
        return controller.getItemAsync(path);
      }
      return ParsePromise.default.as(controller.getItem(path));
    },
    setItem: function (path, value) {
      var controller = CoreManager.getStorageController();
      if (controller.async === 1) {
        throw new Error('Synchronous storage is not supported by the current storage controller');
      }
      return controller.setItem(path, value);
    },
    setItemAsync: function (path, value) {
      var controller = CoreManager.getStorageController();
      if (controller.async === 1) {
        return controller.setItemAsync(path, value);
      }
      return ParsePromise.default.as(controller.setItem(path, value));
    },
    removeItem: function (path) {
      var controller = CoreManager.getStorageController();
      if (controller.async === 1) {
        throw new Error('Synchronous storage is not supported by the current storage controller');
      }
      return controller.removeItem(path);
    },
    removeItemAsync: function (path) {
      var controller = CoreManager.getStorageController();
      if (controller.async === 1) {
        return controller.removeItemAsync(path);
      }
      return ParsePromise.default.as(controller.removeItem(path));
    },
    generatePath: function (path) {
      if (!CoreManager.get('APPLICATION_ID')) {
        throw new Error('You need to call Parse.initialize before using Parse.');
      }
      if (typeof path !== 'string') {
        throw new Error('Tried to get a Storage path that was not a String.');
      }
      if (path[0] === '/') {
        path = path.substr(1);
      }
      return 'Parse/' + CoreManager.get('APPLICATION_ID') + '/' + path;
    },
    _clear: function () {
      var controller = CoreManager.getStorageController();
      if (controller.hasOwnProperty('clear')) {
        controller.clear();
      }
    }
  });
};

var _include = require('./include');

var _include2 = _interopRequireDefault(_include);

function _interopRequireDefault(obj) {
  return obj && obj.__esModule ? obj : { default: obj };
}