'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _weakMap = require('babel-runtime/core-js/weak-map');

var _weakMap2 = _interopRequireDefault(_weakMap);

exports.default = function (CoreManager, Exports) {

  Exports.getState = getState;
  Exports.initializeState = initializeState;
  Exports.removeState = function (obj) {
    var state = getState(obj);
    if (state === null) {
      return null;
    }
    objectState.delete(obj);
    return state;
  };
  Exports.getServerData = getServerData;
  Exports.setServerData = function (obj, attributes) {
    var serverData = initializeState(obj).serverData;
    ObjectStateMutations.setServerData(serverData, attributes);
  };
  Exports.getPendingOps = getPendingOps;
  Exports.setPendingOp = function (obj, attr, op) {
    var pendingOps = initializeState(obj).pendingOps;
    ObjectStateMutations.setPendingOp(pendingOps, attr, op);
  };
  Exports.pushPendingState = function (obj) {
    var pendingOps = initializeState(obj).pendingOps;
    ObjectStateMutations.pushPendingState(pendingOps);
  };
  Exports.popPendingState = function (obj) {
    var pendingOps = initializeState(obj).pendingOps;
    return ObjectStateMutations.popPendingState(pendingOps);
  };
  Exports.mergeFirstPendingState = function (obj) {
    var pendingOps = getPendingOps(obj);
    ObjectStateMutations.mergeFirstPendingState(pendingOps);
  };
  Exports.getObjectCache = function (obj) {
    var state = getState(obj);
    if (state) {
      return state.objectCache;
    }
    return {};
  };
  Exports.estimateAttribute = function (obj, attr) {
    var serverData = getServerData(obj);
    var pendingOps = getPendingOps(obj);
    return ObjectStateMutations.estimateAttribute(serverData, pendingOps, obj.className, obj.id, attr);
  };
  Exports.estimateAttributes = function (obj) {
    var serverData = getServerData(obj);
    var pendingOps = getPendingOps(obj);
    return ObjectStateMutations.estimateAttributes(serverData, pendingOps, obj.className, obj.id);
  };
  Exports.commitServerChanges = function (obj, changes) {
    var state = initializeState(obj);
    ObjectStateMutations.commitServerChanges(state.serverData, state.objectCache, changes);
  };
  Exports.enqueueTask = function (obj, task) {
    var state = initializeState(obj);
    return state.tasks.enqueue(task);
  };
  Exports.duplicateState = function (source, dest) {
    var oldState = initializeState(source);
    var newState = initializeState(dest);
    for (var key in oldState.serverData) {
      newState.serverData[key] = oldState.serverData[key];
    }
    for (var index = 0; index < oldState.pendingOps.length; index++) {
      for (var _key in oldState.pendingOps[index]) {
        newState.pendingOps[index][_key] = oldState.pendingOps[index][_key];
      }
    }
    for (var _key2 in oldState.objectCache) {
      newState.objectCache[_key2] = oldState.objectCache[_key2];
    }
    newState.existed = oldState.existed;
  };
  Exports.clearAllState = function () {
    objectState = new _weakMap2.default();
  };

  var ObjectStateMutations = (0, _include2.default)('./ObjectStateMutations', CoreManager);
  var TaskQueue = (0, _include2.default)('./TaskQueue', CoreManager);

  var objectState = new _weakMap2.default();

  function getState(obj) {
    var classData = objectState.get(obj);
    return classData || null;
  }

  function initializeState(obj, initial) {
    var state = getState(obj);
    if (state) {
      return state;
    }
    if (!initial) {
      initial = {
        serverData: {},
        pendingOps: [{}],
        objectCache: {},
        tasks: new TaskQueue.default(),
        existed: false
      };
    }
    state = initial;
    objectState.set(obj, state);
    return state;
  }

  function getServerData(obj) {
    var state = getState(obj);
    if (state) {
      return state.serverData;
    }
    return {};
  }

  function getPendingOps(obj) {
    var state = getState(obj);
    if (state) {
      return state.pendingOps;
    }
    return [{}];
  }
};

var _include = require('./include');

var _include2 = _interopRequireDefault(_include);

function _interopRequireDefault(obj) {
  return obj && obj.__esModule ? obj : { default: obj };
}