'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof2 = require('babel-runtime/helpers/typeof');

var _typeof3 = _interopRequireDefault(_typeof2);

exports.default = function (CoreManager, Exports) {

  Exports.default = decode;

  (0, _include2.default)('./ParseACL', CoreManager);

  var ParseFile = (0, _include2.default)('./ParseFile', CoreManager);
  var ParseGeoPoint = (0, _include2.default)('./ParseGeoPoint', CoreManager);
  var ParsePolygon = (0, _include2.default)('./ParsePolygon', CoreManager);
  var ParseObject = (0, _include2.default)('./ParseObject', CoreManager);
  var ParseOp = (0, _include2.default)('./ParseOp', CoreManager);
  var ParseRelation = (0, _include2.default)('./ParseRelation', CoreManager);

  function decode(value) {
    if (value === null || (typeof value === 'undefined' ? 'undefined' : (0, _typeof3.default)(value)) !== 'object') {
      return value;
    }
    if (Array.isArray(value)) {
      var dup = [];
      value.forEach(function (v, i) {
        dup[i] = decode(v);
      });
      return dup;
    }
    if (typeof value.__op === 'string') {
      return ParseOp.opFromJSON(value);
    }
    if (value.__type === 'Pointer' && value.className) {
      return ParseObject.default.fromJSON(value);
    }
    if (value.__type === 'Object' && value.className) {
      return ParseObject.default.fromJSON(value);
    }
    if (value.__type === 'Relation') {
      // The parent and key fields will be populated by the parent
      var relation = new ParseRelation.default(null, null);
      relation.targetClassName = value.className;
      return relation;
    }
    if (value.__type === 'Date') {
      return new Date(value.iso);
    }
    if (value.__type === 'File') {
      return ParseFile.default.fromJSON(value);
    }
    if (value.__type === 'GeoPoint') {
      return new ParseGeoPoint.default({
        latitude: value.latitude,
        longitude: value.longitude
      });
    }
    if (value.__type === 'Polygon') {
      return new ParsePolygon.default(value.coordinates);
    }
    var copy = {};
    for (var k in value) {
      copy[k] = decode(value[k]);
    }
    return copy;
  }
};

var _include = require('./include');

var _include2 = _interopRequireDefault(_include);

function _interopRequireDefault(obj) {
  return obj && obj.__esModule ? obj : { default: obj };
}