'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof2 = require('babel-runtime/helpers/typeof');

var _typeof3 = _interopRequireDefault(_typeof2);

exports.default = function (CoreManager, Exports) {

  Exports.default =

  /**
   * Return an array of unsaved children, which are either Parse Objects or Files.
   * If it encounters any dirty Objects without Ids, it will throw an exception.
   */
  function (obj, allowDeepUnsaved) {
    var encountered = {
      objects: {},
      files: []
    };
    var identifier = obj.className + ':' + obj._getId();
    encountered.objects[identifier] = obj.dirty() ? obj : true;
    var attributes = obj.attributes;
    for (var attr in attributes) {
      if ((0, _typeof3.default)(attributes[attr]) === 'object') {
        traverse(attributes[attr], encountered, false, !!allowDeepUnsaved);
      }
    }
    var unsaved = [];
    for (var id in encountered.objects) {
      if (id !== identifier && encountered.objects[id] !== true) {
        unsaved.push(encountered.objects[id]);
      }
    }
    return unsaved.concat(encountered.files);
  };

  var ParseFile = (0, _include2.default)('./ParseFile', CoreManager);
  var ParseObject = (0, _include2.default)('./ParseObject', CoreManager);
  var ParseRelation = (0, _include2.default)('./ParseRelation', CoreManager);

  function traverse(obj, encountered, shouldThrow, allowDeepUnsaved) {
    if (obj instanceof ParseObject.default) {
      if (!obj.id && shouldThrow) {
        throw new Error('Cannot create a pointer to an unsaved Object.');
      }
      var identifier = obj.className + ':' + obj._getId();
      if (!encountered.objects[identifier]) {
        encountered.objects[identifier] = obj.dirty() ? obj : true;
        var attributes = obj.attributes;
        for (var attr in attributes) {
          if ((0, _typeof3.default)(attributes[attr]) === 'object') {
            traverse(attributes[attr], encountered, !allowDeepUnsaved, allowDeepUnsaved);
          }
        }
      }
      return;
    }
    if (obj instanceof ParseFile.default) {
      if (!obj.url() && encountered.files.indexOf(obj) < 0) {
        encountered.files.push(obj);
      }
      return;
    }
    if (obj instanceof ParseRelation.default) {
      return;
    }
    if (Array.isArray(obj)) {
      obj.forEach(function (el) {
        if ((typeof el === 'undefined' ? 'undefined' : (0, _typeof3.default)(el)) === 'object') {
          traverse(el, encountered, shouldThrow, allowDeepUnsaved);
        }
      });
    }
    for (var k in obj) {
      if ((0, _typeof3.default)(obj[k]) === 'object') {
        traverse(obj[k], encountered, shouldThrow, allowDeepUnsaved);
      }
    }
  }
};

var _include = require('./include');

var _include2 = _interopRequireDefault(_include);

function _interopRequireDefault(obj) {
  return obj && obj.__esModule ? obj : { default: obj };
}