'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

var _typeof2 = require('babel-runtime/helpers/typeof');

var _typeof3 = _interopRequireDefault(_typeof2);

exports.default = function (CoreManager, Exports) {

  Exports.defaultState = function () {
    return {
      serverData: {},
      pendingOps: [{}],
      objectCache: {},
      tasks: new TaskQueue.default(),
      existed: false
    };
  };
  Exports.setServerData = function (serverData, attributes) {
    for (var _attr in attributes) {
      if (typeof attributes[_attr] !== 'undefined') {
        serverData[_attr] = attributes[_attr];
      } else {
        delete serverData[_attr];
      }
    }
  };
  Exports.setPendingOp = function (pendingOps, attr, op) {
    var last = pendingOps.length - 1;
    if (op) {
      pendingOps[last][attr] = op;
    } else {
      delete pendingOps[last][attr];
    }
  };
  Exports.pushPendingState = function (pendingOps) {
    pendingOps.push({});
  };
  Exports.popPendingState = popPendingState;
  Exports.mergeFirstPendingState = function (pendingOps) {
    var first = popPendingState(pendingOps);
    var next = pendingOps[0];
    for (var _attr2 in first) {
      if (next[_attr2] && first[_attr2]) {
        var merged = next[_attr2].mergeWith(first[_attr2]);
        if (merged) {
          next[_attr2] = merged;
        }
      } else {
        next[_attr2] = first[_attr2];
      }
    }
  };
  Exports.estimateAttribute = function (serverData, pendingOps, className, id, attr) {
    var value = serverData[attr];
    for (var i = 0; i < pendingOps.length; i++) {
      if (pendingOps[i][attr]) {
        if (pendingOps[i][attr] instanceof ParseOp.RelationOp) {
          if (id) {
            value = pendingOps[i][attr].applyTo(value, { className: className, id: id }, attr);
          }
        } else {
          value = pendingOps[i][attr].applyTo(value);
        }
      }
    }
    return value;
  };
  Exports.estimateAttributes = function (serverData, pendingOps, className, id) {
    var data = {};
    var attr = void 0;
    for (attr in serverData) {
      data[attr] = serverData[attr];
    }
    for (var i = 0; i < pendingOps.length; i++) {
      for (attr in pendingOps[i]) {
        if (pendingOps[i][attr] instanceof ParseOp.RelationOp) {
          if (id) {
            data[attr] = pendingOps[i][attr].applyTo(data[attr], { className: className, id: id }, attr);
          }
        } else {
          data[attr] = pendingOps[i][attr].applyTo(data[attr]);
        }
      }
    }
    return data;
  };
  Exports.commitServerChanges = function (serverData, objectCache, changes) {
    for (var _attr3 in changes) {
      var val = changes[_attr3];
      serverData[_attr3] = val;
      if (val && (typeof val === 'undefined' ? 'undefined' : (0, _typeof3.default)(val)) === 'object' && !(val instanceof ParseObject.default) && !(val instanceof ParseFile.default) && !(val instanceof ParseRelation.default)) {
        var json = encode.default(val, false, true);
        objectCache[_attr3] = (0, _stringify2.default)(json);
      }
    }
  };

  var encode = (0, _include2.default)('./encode', CoreManager);
  var ParseFile = (0, _include2.default)('./ParseFile', CoreManager);
  var ParseObject = (0, _include2.default)('./ParseObject', CoreManager);
  (0, _include2.default)('./ParsePromise', CoreManager);

  var ParseRelation = (0, _include2.default)('./ParseRelation', CoreManager);
  var TaskQueue = (0, _include2.default)('./TaskQueue', CoreManager);
  var ParseOp = (0, _include2.default)('./ParseOp', CoreManager);

  function popPendingState(pendingOps) {
    var first = pendingOps.shift();
    if (!pendingOps.length) {
      pendingOps[0] = {};
    }
    return first;
  }
};

var _include = require('./include');

var _include2 = _interopRequireDefault(_include);

function _interopRequireDefault(obj) {
  return obj && obj.__esModule ? obj : { default: obj };
}