'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = createParse;

var _include = require('./include');

var _include2 = _interopRequireDefault(_include);

var _CoreManager = require('./CoreManager');

var _CoreManager2 = _interopRequireDefault(_CoreManager);

function _interopRequireDefault(obj) {
  return obj && obj.__esModule ? obj : { default: obj };
}

/**
 * Copyright (c) 2015-present, Parse, LLC.
 * All rights reserved.
 *
 * This source code is licensed under the BSD-style license found in the
 * LICENSE file in the root directory of this source tree. An additional grant
 * of patent rights can be found in the PATENTS file in the same directory.
 */
function createParse() {

  var CoreManager = new _CoreManager2.default();

  var decode = (0, _include2.default)('./decode', CoreManager);
  var encode = (0, _include2.default)('./encode', CoreManager);
  var InstallationController = (0, _include2.default)('./InstallationController', CoreManager);
  var ParseOp = (0, _include2.default)('./ParseOp', CoreManager);
  var RESTController = (0, _include2.default)('./RESTController', CoreManager);

  /**
   * Contains all Parse API classes and functions.
   * @static
   * @global
   * @class
   * @hideconstructor
   */
  var Parse = {
    /**
     * Call this method first to set up your authentication tokens for Parse.
     * You can get your keys from the Data Browser on parse.com.
     * @param {String} applicationId Your Parse Application ID.
     * @param {String} javaScriptKey (optional) Your Parse JavaScript Key (Not needed for parse-server)
     * @param {String} masterKey (optional) Your Parse Master Key. (Node.js only!)
     * @static
     */
    initialize: function (applicationId, javaScriptKey) {
      Parse._initialize(applicationId, javaScriptKey);
    },
    _initialize: function (applicationId, javaScriptKey, masterKey) {
      CoreManager.set('APPLICATION_ID', applicationId);
      CoreManager.set('JAVASCRIPT_KEY', javaScriptKey);
      CoreManager.set('MASTER_KEY', masterKey);
      CoreManager.set('USE_MASTER_KEY', false);
    },

    /**
     * Call this method to set your AsyncStorage engine
     * Starting Parse@1.11, the ParseSDK do not provide a React AsyncStorage as the ReactNative module
     * is not provided at a stable path and changes over versions.
     * @param {AsyncStorage} storage a react native async storage.
     * @static
     */
    setAsyncStorage: function (storage) {
      CoreManager.setAsyncStorage(storage);
    }
  };

  /** These legacy setters may eventually be deprecated **/
  /** 
   * @member Parse.applicationId
   * @type string
   * @static
   */
  Object.defineProperty(Parse, 'applicationId', {
    get: function () {
      return CoreManager.get('APPLICATION_ID');
    },
    set: function (value) {
      CoreManager.set('APPLICATION_ID', value);
    }
  });

  /** 
   * @member Parse.javaScriptKey
   * @type string
   * @static
   */
  Object.defineProperty(Parse, 'javaScriptKey', {
    get: function () {
      return CoreManager.get('JAVASCRIPT_KEY');
    },
    set: function (value) {
      CoreManager.set('JAVASCRIPT_KEY', value);
    }
  });

  /** 
   * @member Parse.masterKey
   * @type string
   * @static
   */
  Object.defineProperty(Parse, 'masterKey', {
    get: function () {
      return CoreManager.get('MASTER_KEY');
    },
    set: function (value) {
      CoreManager.set('MASTER_KEY', value);
    }
  });

  /** 
   * @member Parse.serverURL
   * @type string
   * @static
   */
  Object.defineProperty(Parse, 'serverURL', {
    get: function () {
      return CoreManager.get('SERVER_URL');
    },
    set: function (value) {
      CoreManager.set('SERVER_URL', value);
    }
  });
  /** 
   * @member Parse.liveQueryServerURL
   * @type string
   * @static
   */
  Object.defineProperty(Parse, 'liveQueryServerURL', {
    get: function () {
      return CoreManager.get('LIVEQUERY_SERVER_URL');
    },
    set: function (value) {
      CoreManager.set('LIVEQUERY_SERVER_URL', value);
    }
  });

  /**
   * @member Parse.datonisURL
   * @type string
   * @static
   */
  Object.defineProperty(Parse, 'datonisURL', {
    get: function () {
      return CoreManager.get('DATONIS_URL');
    },
    set: function (value) {
      CoreManager.set('DATONIS_URL', value);
    }
  });
  /* End setters */

  Parse.ACL = (0, _include2.default)('./ParseACL', CoreManager).default;
  Parse.Analytics = (0, _include2.default)('./Analytics', CoreManager);
  Parse.Cloud = (0, _include2.default)('./Cloud', CoreManager);
  Parse.CoreManager = CoreManager;
  Parse.Config = (0, _include2.default)('./ParseConfig', CoreManager).default;
  Parse.Error = (0, _include2.default)('./ParseError', CoreManager).default;
  Parse.FacebookUtils = (0, _include2.default)('./FacebookUtils', CoreManager).default;
  Parse.File = (0, _include2.default)('./ParseFile', CoreManager).default;
  Parse.GeoPoint = (0, _include2.default)('./ParseGeoPoint', CoreManager).default;
  Parse.Polygon = (0, _include2.default)('./ParsePolygon', CoreManager).default;
  Parse.Installation = (0, _include2.default)('./ParseInstallation', CoreManager).default;
  Parse.Object = (0, _include2.default)('./ParseObject', CoreManager).default;
  Parse.Op = {
    Set: ParseOp.SetOp,
    Unset: ParseOp.UnsetOp,
    Increment: ParseOp.IncrementOp,
    Add: ParseOp.AddOp,
    Remove: ParseOp.RemoveOp,
    AddUnique: ParseOp.AddUniqueOp,
    Relation: ParseOp.RelationOp
  };
  Parse.Promise = (0, _include2.default)('./ParsePromise', CoreManager).default;
  Parse.Push = (0, _include2.default)('./Push', CoreManager);
  Parse.Query = (0, _include2.default)('./ParseQuery', CoreManager).default;
  Parse.Relation = (0, _include2.default)('./ParseRelation', CoreManager).default;
  Parse.Role = (0, _include2.default)('./ParseRole', CoreManager).default;
  Parse.Schema = (0, _include2.default)('./ParseSchema', CoreManager).default;
  Parse.Session = (0, _include2.default)('./ParseSession', CoreManager).default;
  Parse.Storage = (0, _include2.default)('./Storage', CoreManager);
  Parse.User = (0, _include2.default)('./ParseUser', CoreManager).default;
  Parse.LiveQuery = (0, _include2.default)('./ParseLiveQuery', CoreManager).default;
  Parse.LiveQueryClient = (0, _include2.default)('./LiveQueryClient', CoreManager).default;
  //Setting Datonis object
  Parse.Datonis = (0, _include2.default)('./Datonis', CoreManager).default;
  Parse._request = function () {
    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    return CoreManager.getRESTController().request.apply(null, args);
  };
  Parse._ajax = function () {
    for (var _len2 = arguments.length, args = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
      args[_key2] = arguments[_key2];
    }

    return CoreManager.getRESTController().ajax.apply(null, args);
  };
  // We attempt to match the signatures of the legacy versions of these methods
  Parse._decode = function (_, value) {
    return decode.default(value);
  };
  Parse._encode = function (value, _, disallowObjects) {
    return encode.default(value, disallowObjects);
  };
  Parse._getInstallationId = function () {
    return CoreManager.getInstallationController().currentInstallationId();
  };

  CoreManager.setInstallationController(InstallationController);
  CoreManager.setRESTController(RESTController);

  Parse.initialize = Parse._initialize;
  Parse.Cloud = Parse.Cloud || {};
  Parse.Cloud.useMasterKey = function () {
    CoreManager.set('USE_MASTER_KEY', true);
  };

  Parse.Hooks = (0, _include2.default)('./ParseHooks', CoreManager);
  Parse.Tasks = (0, _include2.default)('./ParseTasks', CoreManager);

  // For legacy requires, of the form `var Parse = include('@datonis/parse').Parse`
  Parse.Parse = Parse;

  return Parse;
}