'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

exports.default = function (CoreManager, Exports) {

  Exports.getFunctions = function () {
    return CoreManager.getHooksController().get("functions");
  };
  Exports.getTriggers = function () {
    return CoreManager.getHooksController().get("triggers");
  };
  Exports.getFunction = function (name) {
    return CoreManager.getHooksController().get("functions", name);
  };
  Exports.getTrigger = function (className, triggerName) {
    return CoreManager.getHooksController().get("triggers", className, triggerName);
  };
  Exports.createFunction = function (functionName, url) {
    return create({ functionName: functionName, url: url });
  };
  Exports.createTrigger = function (className, triggerName, url) {
    return create({ className: className, triggerName: triggerName, url: url });
  };
  Exports.create = create;
  Exports.updateFunction = function (functionName, url) {
    return update({ functionName: functionName, url: url });
  };
  Exports.updateTrigger = function (className, triggerName, url) {
    return update({ className: className, triggerName: triggerName, url: url });
  };
  Exports.update = update;
  Exports.removeFunction = function (functionName) {
    return remove({ functionName: functionName });
  };
  Exports.removeTrigger = function (className, triggerName) {
    return remove({ className: className, triggerName: triggerName });
  };
  Exports.remove = remove;

  var decode = (0, _include2.default)('./decode', CoreManager);
  (0, _include2.default)('./encode', CoreManager);

  var ParseError = (0, _include2.default)('./ParseError', CoreManager);
  var ParsePromise = (0, _include2.default)('./ParsePromise', CoreManager);

  function create(hook) {
    return CoreManager.getHooksController().create(hook);
  }

  function update(hook) {
    return CoreManager.getHooksController().update(hook);
  }

  function remove(hook) {
    return CoreManager.getHooksController().remove(hook);
  }

  CoreManager.setHooksController({
    get: function (type, functionName, triggerName) {
      var url = "/hooks/" + type;
      if (functionName) {
        url += "/" + functionName;
        if (triggerName) {
          url += "/" + triggerName;
        }
      }
      return this.sendRequest("GET", url);
    },
    create: function (hook) {
      var url;
      if (hook.functionName && hook.url) {
        url = "/hooks/functions";
      } else if (hook.className && hook.triggerName && hook.url) {
        url = "/hooks/triggers";
      } else {
        return _promise2.default.reject({ error: 'invalid hook declaration', code: 143 });
      }
      return this.sendRequest("POST", url, hook);
    },
    remove: function (hook) {
      var url;
      if (hook.functionName) {
        url = "/hooks/functions/" + hook.functionName;
        delete hook.functionName;
      } else if (hook.className && hook.triggerName) {
        url = "/hooks/triggers/" + hook.className + "/" + hook.triggerName;
        delete hook.className;
        delete hook.triggerName;
      } else {
        return _promise2.default.reject({ error: 'invalid hook declaration', code: 143 });
      }
      return this.sendRequest("PUT", url, { "__op": "Delete" });
    },
    update: function (hook) {
      var url;
      if (hook.functionName && hook.url) {
        url = "/hooks/functions/" + hook.functionName;
        delete hook.functionName;
      } else if (hook.className && hook.triggerName && hook.url) {
        url = "/hooks/triggers/" + hook.className + "/" + hook.triggerName;
        delete hook.className;
        delete hook.triggerName;
      } else {
        return _promise2.default.reject({ error: 'invalid hook declaration', code: 143 });
      }
      return this.sendRequest('PUT', url, hook);
    },
    sendRequest: function (method, url, body) {
      return CoreManager.getRESTController().request(method, url, body, { useMasterKey: true }).then(function (res) {
        var decoded = decode.default(res);
        if (decoded) {
          return ParsePromise.default.as(decoded);
        }
        return ParsePromise.default.error(new ParseError.default(ParseError.default.INVALID_JSON, 'The server returned an invalid response.'));
      });
    }
  });
};

var _include = require('./include');

var _include2 = _interopRequireDefault(_include);

function _interopRequireDefault(obj) {
  return obj && obj.__esModule ? obj : { default: obj };
}