'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof2 = require('babel-runtime/helpers/typeof');

var _typeof3 = _interopRequireDefault(_typeof2);

exports.default = function (CoreManager, Exports) {

  Exports.send =

  /**
   * Contains functions to deal with Push in Parse.
   * @class Parse.Push
   * @static
   * @hideconstructor
   */

  /**
   * Sends a push notification.
   * @method send
   * @name Parse.Push.send
   * @param {Object} data -  The data of the push notification.  Valid fields
   * are:
   *   <ol>
   *     <li>channels - An Array of channels to push to.</li>
   *     <li>push_time - A Date object for when to send the push.</li>
   *     <li>expiration_time -  A Date object for when to expire
   *         the push.</li>
   *     <li>expiration_interval - The seconds from now to expire the push.</li>
   *     <li>where - A Parse.Query over Parse.Installation that is used to match
   *         a set of installations to push to.</li>
   *     <li>data - The data to send as part of the push</li>
   *   <ol>
   * @param {Object} options An object that has an optional success function,
   * that takes no arguments and will be called on a successful push, and
   * an error function that takes a Parse.Error and will be called if the push
   * failed.
   * @return {Parse.Promise} A promise that is fulfilled when the push request
   *     completes.
   */
  function (data, options) {
    options = options || {};

    if (data.where && data.where instanceof ParseQuery.default) {
      data.where = data.where.toJSON().where;
    }

    if (data.push_time && (0, _typeof3.default)(data.push_time) === 'object') {
      data.push_time = data.push_time.toJSON();
    }

    if (data.expiration_time && (0, _typeof3.default)(data.expiration_time) === 'object') {
      data.expiration_time = data.expiration_time.toJSON();
    }

    if (data.expiration_time && data.expiration_interval) {
      throw new Error('expiration_time and expiration_interval cannot both be set.');
    }

    return CoreManager.getPushController().send(data, {
      useMasterKey: options.useMasterKey
    })._thenRunCallbacks(options);
  };

  var ParseQuery = (0, _include2.default)('./ParseQuery', CoreManager);

  CoreManager.setPushController({
    send: function (data, options) {
      var RESTController = CoreManager.getRESTController();

      var request = RESTController.request('POST', 'push', data, { useMasterKey: !!options.useMasterKey });

      return request._thenRunCallbacks(options);
    }
  });
};

var _include = require('./include');

var _include2 = _interopRequireDefault(_include);

function _interopRequireDefault(obj) {
  return obj && obj.__esModule ? obj : { default: obj };
}