'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _assign = require('babel-runtime/core-js/object/assign');

var _assign2 = _interopRequireDefault(_assign);

var _typeof2 = require('babel-runtime/helpers/typeof');

var _typeof3 = _interopRequireDefault(_typeof2);

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

exports.default = function (CoreManager, Exports) {

  var ParseError = (0, _include2.default)('./ParseError', CoreManager);
  var ParsePromise = (0, _include2.default)('./ParsePromise', CoreManager);
  (0, _include2.default)('./Storage', CoreManager);


  var XHR = null;
  if (typeof XMLHttpRequest !== 'undefined') {
    XHR = XMLHttpRequest;
  }

  XHR = require('xmlhttprequest').XMLHttpRequest;


  var useXDomainRequest = false;
  if (typeof XDomainRequest !== 'undefined' && !('withCredentials' in new XMLHttpRequest())) {
    useXDomainRequest = true;
  }

  function ajaxIE9(method, url, data) {
    var promise = new ParsePromise.default();
    var xdr = new XDomainRequest();
    xdr.onload = function () {
      var response;
      try {
        response = JSON.parse(xdr.responseText);
      } catch (e) {
        promise.reject(e);
      }
      if (response) {
        promise.resolve(response);
      }
    };
    xdr.onerror = xdr.ontimeout = function () {
      // Let's fake a real error message.
      var fakeResponse = {
        responseText: (0, _stringify2.default)({
          code: ParseError.default.X_DOMAIN_REQUEST,
          error: 'IE\'s XDomainRequest does not supply error info.'
        })
      };
      promise.reject(fakeResponse);
    };
    xdr.onprogress = function () {};
    xdr.open(method, url);
    xdr.send(data);
    return promise;
  }

  var RESTController = {
    ajax: function (method, url, data, headers) {
      if (useXDomainRequest) {
        return ajaxIE9(method, url, data, headers);
      }

      var promise = new ParsePromise.default();
      var attempts = 0;

      (function dispatch() {
        if (XHR == null) {
          throw new Error('Cannot make a request: No definition of XMLHttpRequest was found.');
        }
        var handled = false;
        var xhr = new XHR();

        xhr.onreadystatechange = function () {
          if (xhr.readyState !== 4 || handled) {
            return;
          }
          handled = true;

          if (xhr.status >= 200 && xhr.status < 300) {
            var response;
            try {
              response = JSON.parse(xhr.responseText);
            } catch (e) {
              promise.reject(e.toString());
            }
            if (response) {
              promise.resolve(response, xhr.status, xhr);
            }
          } else if (xhr.status >= 500 || xhr.status === 0) {
            // retry on 5XX or node-xmlhttprequest error
            if (++attempts < CoreManager.get('REQUEST_ATTEMPT_LIMIT')) {
              // Exponentially-growing random delay
              var delay = Math.round(Math.random() * 125 * Math.pow(2, attempts));
              setTimeout(dispatch, delay);
            } else if (xhr.status === 0) {
              promise.reject('Unable to connect to the Parse API');
            } else {
              // After the retry limit is reached, fail
              promise.reject(xhr);
            }
          } else {
            promise.reject(xhr);
          }
        };

        headers = headers || {};
        if (typeof headers['Content-Type'] !== 'string') {
          headers['Content-Type'] = 'text/plain'; // Avoid pre-flight
        }
        if (CoreManager.get('IS_NODE')) {
          headers['User-Agent'] = 'Parse/' + CoreManager.get('VERSION') + ' (NodeJS ' + process.versions.node + ')';
        }

        xhr.open(method, url, true);
        for (var h in headers) {
          xhr.setRequestHeader(h, headers[h]);
        }
        xhr.send(data);
      })();

      return promise;
    },
    request: function (method, path, data, options) {
      options = options || {};
      var url = CoreManager.get('SERVER_URL');
      if (url[url.length - 1] !== '/') {
        url += '/';
      }
      url += path;

      var payload = {};
      if (data && (typeof data === 'undefined' ? 'undefined' : (0, _typeof3.default)(data)) === 'object') {
        for (var k in data) {
          payload[k] = data[k];
        }
      }

      if (method !== 'POST') {
        payload._method = method;
        method = 'POST';
      }

      payload._ApplicationId = CoreManager.get('APPLICATION_ID');
      var jsKey = CoreManager.get('JAVASCRIPT_KEY');
      if (jsKey) {
        payload._JavaScriptKey = jsKey;
      }
      payload._ClientVersion = CoreManager.get('VERSION');

      var useMasterKey = options.useMasterKey;
      if (typeof useMasterKey === 'undefined') {
        useMasterKey = CoreManager.get('USE_MASTER_KEY');
      }
      if (useMasterKey) {
        if (CoreManager.get('MASTER_KEY')) {
          delete payload._JavaScriptKey;
          payload._MasterKey = CoreManager.get('MASTER_KEY');
        } else {
          throw new Error('Cannot use the Master Key, it has not been provided.');
        }
      }

      if (CoreManager.get('FORCE_REVOCABLE_SESSION')) {
        payload._RevocableSession = '1';
      }

      var installationId = options.installationId;
      var installationIdPromise;
      if (installationId && typeof installationId === 'string') {
        installationIdPromise = ParsePromise.default.as(installationId);
      } else {
        var installationController = CoreManager.getInstallationController();
        installationIdPromise = installationController.currentInstallationId();
      }

      return installationIdPromise.then(function (iid) {
        payload._InstallationId = iid;
        var userController = CoreManager.getUserController();
        if (options && typeof options.sessionToken === 'string') {
          return ParsePromise.default.as(options.sessionToken);
        } else if (userController) {
          return userController.currentUserAsync().then(function (user) {
            if (user) {
              return ParsePromise.default.as(user.getSessionToken());
            }
            return ParsePromise.default.as(null);
          });
        }
        return ParsePromise.default.as(null);
      }).then(function (token) {
        if (token) {
          payload._SessionToken = token;
        }

        var payloadString = (0, _stringify2.default)(payload);

        return RESTController.ajax(method, url, payloadString);
      }).then(null, function (response) {
        // Transform the error into an instance of ParseError by trying to parse
        // the error string as JSON
        var error;
        if (response && response.responseText) {
          try {
            var errorJSON = JSON.parse(response.responseText);
            error = new ParseError.default(errorJSON.code, errorJSON.error);
          } catch (e) {
            // If we fail to parse the error text, that's okay.
            error = new ParseError.default(ParseError.default.INVALID_JSON, 'Received an error with invalid JSON from Parse: ' + response.responseText);
          }
        } else {
          error = new ParseError.default(ParseError.default.CONNECTION_FAILED, 'XMLHttpRequest failed: ' + (0, _stringify2.default)(response));
        }

        return ParsePromise.default.error(error);
      });
    },
    _setXHR: function (xhr) {
      XHR = xhr;
    }
  };

  (0, _assign2.default)(Exports, RESTController);
};

var _include = require('./include');

var _include2 = _interopRequireDefault(_include);

function _interopRequireDefault(obj) {
  return obj && obj.__esModule ? obj : { default: obj };
}