'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

exports.default = function (CoreManager, Exports) {

  Exports.getState = getState;
  Exports.initializeState = initializeState;
  Exports.removeState = function (obj) {
    var state = getState(obj);
    if (state === null) {
      return null;
    }
    delete objectState[obj.className][obj.id];
    return state;
  };
  Exports.getServerData = getServerData;
  Exports.setServerData = function (obj, attributes) {
    var serverData = initializeState(obj).serverData;
    ObjectStateMutations.setServerData(serverData, attributes);
  };
  Exports.getPendingOps = getPendingOps;
  Exports.setPendingOp = function (obj, attr, op) {
    var pendingOps = initializeState(obj).pendingOps;
    ObjectStateMutations.setPendingOp(pendingOps, attr, op);
  };
  Exports.pushPendingState = function (obj) {
    var pendingOps = initializeState(obj).pendingOps;
    ObjectStateMutations.pushPendingState(pendingOps);
  };
  Exports.popPendingState = function (obj) {
    var pendingOps = initializeState(obj).pendingOps;
    return ObjectStateMutations.popPendingState(pendingOps);
  };
  Exports.mergeFirstPendingState = function (obj) {
    var pendingOps = getPendingOps(obj);
    ObjectStateMutations.mergeFirstPendingState(pendingOps);
  };
  Exports.getObjectCache = function (obj) {
    var state = getState(obj);
    if (state) {
      return state.objectCache;
    }
    return {};
  };
  Exports.estimateAttribute = function (obj, attr) {
    var serverData = getServerData(obj);
    var pendingOps = getPendingOps(obj);
    return ObjectStateMutations.estimateAttribute(serverData, pendingOps, obj.className, obj.id, attr);
  };
  Exports.estimateAttributes = function (obj) {
    var serverData = getServerData(obj);
    var pendingOps = getPendingOps(obj);
    return ObjectStateMutations.estimateAttributes(serverData, pendingOps, obj.className, obj.id);
  };
  Exports.commitServerChanges = function (obj, changes) {
    var state = initializeState(obj);
    ObjectStateMutations.commitServerChanges(state.serverData, state.objectCache, changes);
  };
  Exports.enqueueTask = function (obj, task) {
    var state = initializeState(obj);
    return state.tasks.enqueue(task);
  };
  Exports.clearAllState = function () {
    objectState = {};
  };
  Exports.duplicateState = function (source, dest) {
    dest.id = source.id;
  };

  var ObjectStateMutations = (0, _include2.default)('./ObjectStateMutations', CoreManager);

  var objectState = {};

  function getState(obj) {
    var classData = objectState[obj.className];
    if (classData) {
      return classData[obj.id] || null;
    }
    return null;
  }

  function initializeState(obj, initial) {
    var state = getState(obj);
    if (state) {
      return state;
    }
    if (!objectState[obj.className]) {
      objectState[obj.className] = {};
    }
    if (!initial) {
      initial = ObjectStateMutations.defaultState();
    }
    state = objectState[obj.className][obj.id] = initial;
    return state;
  }

  function getServerData(obj) {
    var state = getState(obj);
    if (state) {
      return state.serverData;
    }
    return {};
  }

  function getPendingOps(obj) {
    var state = getState(obj);
    if (state) {
      return state.pendingOps;
    }
    return [{}];
  }
};

var _include = require('./include');

var _include2 = _interopRequireDefault(_include);

function _interopRequireDefault(obj) {
  return obj && obj.__esModule ? obj : { default: obj };
}