'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof2 = require('babel-runtime/helpers/typeof');

var _typeof3 = _interopRequireDefault(_typeof2);

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

exports.default = function (CoreManager, Exports) {

  Exports.default = function (value, disallowObjects, forcePointers, seen) {
    return encode(value, !!disallowObjects, !!forcePointers, seen || []);
  };

  var ParseACL = (0, _include2.default)('./ParseACL', CoreManager);
  var ParseFile = (0, _include2.default)('./ParseFile', CoreManager);
  var ParseGeoPoint = (0, _include2.default)('./ParseGeoPoint', CoreManager);
  var ParsePolygon = (0, _include2.default)('./ParsePolygon', CoreManager);
  var ParseObject = (0, _include2.default)('./ParseObject', CoreManager);
  var ParseOp = (0, _include2.default)('./ParseOp', CoreManager);
  var ParseRelation = (0, _include2.default)('./ParseRelation', CoreManager);

  var toString = Object.prototype.toString;

  function encode(value, disallowObjects, forcePointers, seen) {
    if (value instanceof ParseObject.default) {
      if (disallowObjects) {
        throw new Error('Parse Objects not allowed here');
      }
      var seenEntry = value.id ? value.className + ':' + value.id : value;
      if (forcePointers || !seen || seen.indexOf(seenEntry) > -1 || value.dirty() || (0, _keys2.default)(value._getServerData()).length < 1) {
        return value.toPointer();
      }
      seen = seen.concat(seenEntry);
      return value._toFullJSON(seen);
    }
    if (value instanceof ParseOp.Op || value instanceof ParseACL.default || value instanceof ParseGeoPoint.default || value instanceof ParsePolygon.default || value instanceof ParseRelation.default) {
      return value.toJSON();
    }
    if (value instanceof ParseFile.default) {
      if (!value.url()) {
        throw new Error('Tried to encode an unsaved file.');
      }
      return value.toJSON();
    }
    if (toString.call(value) === '[object Date]') {
      if (isNaN(value)) {
        throw new Error('Tried to encode an invalid date.');
      }
      return { __type: 'Date', iso: value.toJSON() };
    }
    if (toString.call(value) === '[object RegExp]' && typeof value.source === 'string') {
      return value.source;
    }

    if (Array.isArray(value)) {
      return value.map(function (v) {
        return encode(v, disallowObjects, forcePointers, seen);
      });
    }

    if (value && (typeof value === 'undefined' ? 'undefined' : (0, _typeof3.default)(value)) === 'object') {
      var output = {};
      for (var k in value) {
        output[k] = encode(value[k], disallowObjects, forcePointers, seen);
      }
      return output;
    }

    return value;
  }
};

var _include = require('./include');

var _include2 = _interopRequireDefault(_include);

function _interopRequireDefault(obj) {
  return obj && obj.__esModule ? obj : { default: obj };
}