

import include from './include'; /**
                                  * Copyright (c) 2015-present, Parse, LLC.
                                  * All rights reserved.
                                  *
                                  * This source code is licensed under the BSD-style license found in the
                                  * LICENSE file in the root directory of this source tree. An additional grant
                                  * of patent rights can be found in the PATENTS file in the same directory.
                                  *
                                  * 
                                  */

export default function (CoreManager, Exports) {

  Exports.defaultState = function () {
    return {
      serverData: {},
      pendingOps: [{}],
      objectCache: {},
      tasks: new TaskQueue.default(),
      existed: false
    };
  };
  Exports.setServerData = function (serverData, attributes) {
    for (let attr in attributes) {
      if (typeof attributes[attr] !== 'undefined') {
        serverData[attr] = attributes[attr];
      } else {
        delete serverData[attr];
      }
    }
  };
  Exports.setPendingOp = function (pendingOps, attr, op) {
    let last = pendingOps.length - 1;
    if (op) {
      pendingOps[last][attr] = op;
    } else {
      delete pendingOps[last][attr];
    }
  };
  Exports.pushPendingState = function (pendingOps) {
    pendingOps.push({});
  };
  Exports.popPendingState = popPendingState;
  Exports.mergeFirstPendingState = function (pendingOps) {
    let first = popPendingState(pendingOps);
    let next = pendingOps[0];
    for (let attr in first) {
      if (next[attr] && first[attr]) {
        let merged = next[attr].mergeWith(first[attr]);
        if (merged) {
          next[attr] = merged;
        }
      } else {
        next[attr] = first[attr];
      }
    }
  };
  Exports.estimateAttribute = function (serverData, pendingOps, className, id, attr) {
    let value = serverData[attr];
    for (let i = 0; i < pendingOps.length; i++) {
      if (pendingOps[i][attr]) {
        if (pendingOps[i][attr] instanceof ParseOp.RelationOp) {
          if (id) {
            value = pendingOps[i][attr].applyTo(value, { className: className, id: id }, attr);
          }
        } else {
          value = pendingOps[i][attr].applyTo(value);
        }
      }
    }
    return value;
  };
  Exports.estimateAttributes = function (serverData, pendingOps, className, id) {
    let data = {};

    for (var attr in serverData) {
      data[attr] = serverData[attr];
    }
    for (let i = 0; i < pendingOps.length; i++) {
      for (attr in pendingOps[i]) {
        if (pendingOps[i][attr] instanceof ParseOp.RelationOp) {
          if (id) {
            data[attr] = pendingOps[i][attr].applyTo(data[attr], { className: className, id: id }, attr);
          }
        } else {
          data[attr] = pendingOps[i][attr].applyTo(data[attr]);
        }
      }
    }
    return data;
  };
  Exports.commitServerChanges = function (serverData, objectCache, changes) {
    for (let attr in changes) {
      let val = changes[attr];
      serverData[attr] = val;
      if (val && typeof val === 'object' && !(val instanceof ParseObject.default) && !(val instanceof ParseFile.default) && !(val instanceof ParseRelation.default)) {
        let json = encode.default(val, false, true);
        objectCache[attr] = JSON.stringify(json);
      }
    }
  };

  const encode = include('./encode', CoreManager);
  const ParseFile = include('./ParseFile', CoreManager);
  const ParseObject = include('./ParseObject', CoreManager);
  include('./ParsePromise', CoreManager);

  const ParseRelation = include('./ParseRelation', CoreManager);
  const TaskQueue = include('./TaskQueue', CoreManager);
  const ParseOp = include('./ParseOp', CoreManager);

  function popPendingState(pendingOps) {
    let first = pendingOps.shift();
    if (!pendingOps.length) {
      pendingOps[0] = {};
    }
    return first;
  }
}