import include from './include';
export default function (CoreManager, Exports) {

  Exports.getFunctions = function () {
    return CoreManager.getHooksController().get("functions");
  };
  Exports.getTriggers = function () {
    return CoreManager.getHooksController().get("triggers");
  };
  Exports.getFunction = function (name) {
    return CoreManager.getHooksController().get("functions", name);
  };
  Exports.getTrigger = function (className, triggerName) {
    return CoreManager.getHooksController().get("triggers", className, triggerName);
  };
  Exports.createFunction = function (functionName, url) {
    return create({ functionName: functionName, url: url });
  };
  Exports.createTrigger = function (className, triggerName, url) {
    return create({ className: className, triggerName: triggerName, url: url });
  };
  Exports.create = create;
  Exports.updateFunction = function (functionName, url) {
    return update({ functionName: functionName, url: url });
  };
  Exports.updateTrigger = function (className, triggerName, url) {
    return update({ className: className, triggerName: triggerName, url: url });
  };
  Exports.update = update;
  Exports.removeFunction = function (functionName) {
    return remove({ functionName: functionName });
  };
  Exports.removeTrigger = function (className, triggerName) {
    return remove({ className: className, triggerName: triggerName });
  };
  Exports.remove = remove;

  const decode = include('./decode', CoreManager);
  include('./encode', CoreManager);

  const ParseError = include('./ParseError', CoreManager);
  const ParsePromise = include('./ParsePromise', CoreManager);

  function create(hook) {
    return CoreManager.getHooksController().create(hook);
  }

  function update(hook) {
    return CoreManager.getHooksController().update(hook);
  }

  function remove(hook) {
    return CoreManager.getHooksController().remove(hook);
  }

  var DefaultController = {

    get(type, functionName, triggerName) {
      var url = "/hooks/" + type;
      if (functionName) {
        url += "/" + functionName;
        if (triggerName) {
          url += "/" + triggerName;
        }
      }
      return this.sendRequest("GET", url);
    },

    create(hook) {
      var url;
      if (hook.functionName && hook.url) {
        url = "/hooks/functions";
      } else if (hook.className && hook.triggerName && hook.url) {
        url = "/hooks/triggers";
      } else {
        return Promise.reject({ error: 'invalid hook declaration', code: 143 });
      }
      return this.sendRequest("POST", url, hook);
    },

    remove(hook) {
      var url;
      if (hook.functionName) {
        url = "/hooks/functions/" + hook.functionName;
        delete hook.functionName;
      } else if (hook.className && hook.triggerName) {
        url = "/hooks/triggers/" + hook.className + "/" + hook.triggerName;
        delete hook.className;
        delete hook.triggerName;
      } else {
        return Promise.reject({ error: 'invalid hook declaration', code: 143 });
      }
      return this.sendRequest("PUT", url, { "__op": "Delete" });
    },

    update(hook) {
      var url;
      if (hook.functionName && hook.url) {
        url = "/hooks/functions/" + hook.functionName;
        delete hook.functionName;
      } else if (hook.className && hook.triggerName && hook.url) {
        url = "/hooks/triggers/" + hook.className + "/" + hook.triggerName;
        delete hook.className;
        delete hook.triggerName;
      } else {
        return Promise.reject({ error: 'invalid hook declaration', code: 143 });
      }
      return this.sendRequest('PUT', url, hook);
    },

    sendRequest(method, url, body) {
      return CoreManager.getRESTController().request(method, url, body, { useMasterKey: true }).then(res => {
        var decoded = decode.default(res);
        if (decoded) {
          return ParsePromise.default.as(decoded);
        }
        return ParsePromise.default.error(new ParseError.default(ParseError.default.INVALID_JSON, 'The server returned an invalid response.'));
      });
    }
  };

  CoreManager.setHooksController(DefaultController);
}