import include from './include';

export default function (CoreManager, Exports) {

  Exports.getFunctions = function () {
    return CoreManager.getTasksController().get("functions");
  };
  Exports.getTriggers = function () {
    return CoreManager.getTasksController().get("triggers");
  };
  Exports.getFunction = function (name) {
    return CoreManager.getTasksController().get("functions", name);
  };
  Exports.getTrigger = function (className, triggerName) {
    return CoreManager.getTasksController().get("triggers", className, triggerName);
  };
  Exports.createFunction = function (functionName, taskDefinition, taskRepeat) {
    return create({ functionName: functionName, taskDefinition: taskDefinition, taskRepeat: taskRepeat });
  };
  Exports.createTrigger = function (className, triggerName, taskDefinition, taskRepeat) {
    return create({ className: className, triggerName: triggerName, taskDefinition: taskDefinition, taskRepeat: taskRepeat });
  };
  Exports.create = create;
  Exports.updateFunction = function (functionName, taskDefinition, taskRepeat) {
    return update({ functionName: functionName, taskDefinition: taskDefinition, taskRepeat: taskRepeat });
  };
  Exports.updateTrigger = function (className, triggerName, taskDefinition, taskRepeat) {
    return update({ className: className, triggerName: triggerName, taskDefinition: taskDefinition, taskRepeat: taskRepeat });
  };
  Exports.update = update;
  Exports.removeFunction = function (functionName) {
    return remove({ functionName: functionName });
  };
  Exports.removeTrigger = function (className, triggerName) {
    return remove({ className: className, triggerName: triggerName });
  };
  Exports.remove = remove;

  const decode = include('./decode', CoreManager);
  include('./encode', CoreManager);

  const ParseError = include('./ParseError', CoreManager);
  const ParsePromise = include('./ParsePromise', CoreManager);

  function create(task) {
    return CoreManager.getTasksController().create(task);
  }

  function update(task) {
    return CoreManager.getTasksController().update(task);
  }

  function remove(task) {
    return CoreManager.getTasksController().remove(task);
  }

  var DefaultController = {

    get(type, functionName, triggerName) {
      var url = "/tasks/" + type;
      if (functionName) {
        url += "/" + functionName;
        if (triggerName) {
          url += "/" + triggerName;
        }
      }
      return this.sendRequest("GET", url);
    },

    create(task) {
      var url;
      if (task.functionName && task.taskDefinition) {
        url = "/tasks/functions";
      } else if (task.className && task.triggerName && task.taskDefinition) {
        url = "/tasks/triggers";
      } else {
        return Promise.reject({ error: 'invalid task declaration', code: 143 });
      }
      return this.sendRequest("POST", url, task);
    },

    remove(task) {
      var url;
      if (task.functionName) {
        url = "/tasks/functions/" + task.functionName;
        delete task.functionName;
      } else if (task.className && task.triggerName) {
        url = "/tasks/triggers/" + task.className + "/" + task.triggerName;
        delete task.className;
        delete task.triggerName;
      } else {
        return Promise.reject({ error: 'invalid task declaration', code: 143 });
      }
      return this.sendRequest("PUT", url, { "__op": "Delete" });
    },

    update(task) {
      var url;
      if (task.functionName && task.taskDefinition) {
        url = "/tasks/functions/" + task.functionName;
        delete task.functionName;
      } else if (task.className && task.triggerName && task.taskDefinition) {
        url = "/tasks/triggers/" + task.className + "/" + task.triggerName;
        delete task.className;
        delete task.triggerName;
      } else {
        return Promise.reject({ error: 'invalid task declaration', code: 143 });
      }
      return this.sendRequest('PUT', url, task);
    },

    sendRequest(method, url, body) {
      return CoreManager.getRESTController().request(method, url, body, { useMasterKey: true }).then(res => {
        var decoded = decode.default(res);
        if (decoded) {
          return ParsePromise.default.as(decoded);
        }
        return ParsePromise.default.error(new ParseError.default(ParseError.default.INVALID_JSON, 'The server returned an invalid response.'));
      });
    }
  };

  CoreManager.setTasksController(DefaultController);
}