/**
 * Copyright (c) 2015-present, Parse, LLC.
 * All rights reserved.
 *
 * This source code is licensed under the BSD-style license found in the
 * LICENSE file in the root directory of this source tree. An additional grant
 * of patent rights can be found in the PATENTS file in the same directory.
 *
 * 
 */
import include from './include';
export default function (CoreManager, Exports) {

  Exports.getState = getState;
  Exports.initializeState = initializeState;
  Exports.removeState = function (obj) {
    let state = getState(obj);
    if (state === null) {
      return null;
    }
    delete objectState[obj.className][obj.id];
    return state;
  };
  Exports.getServerData = getServerData;
  Exports.setServerData = function (obj, attributes) {
    let serverData = initializeState(obj).serverData;
    ObjectStateMutations.setServerData(serverData, attributes);
  };
  Exports.getPendingOps = getPendingOps;
  Exports.setPendingOp = function (obj, attr, op) {
    let pendingOps = initializeState(obj).pendingOps;
    ObjectStateMutations.setPendingOp(pendingOps, attr, op);
  };
  Exports.pushPendingState = function (obj) {
    let pendingOps = initializeState(obj).pendingOps;
    ObjectStateMutations.pushPendingState(pendingOps);
  };
  Exports.popPendingState = function (obj) {
    let pendingOps = initializeState(obj).pendingOps;
    return ObjectStateMutations.popPendingState(pendingOps);
  };
  Exports.mergeFirstPendingState = function (obj) {
    let pendingOps = getPendingOps(obj);
    ObjectStateMutations.mergeFirstPendingState(pendingOps);
  };
  Exports.getObjectCache = function (obj) {
    let state = getState(obj);
    if (state) {
      return state.objectCache;
    }
    return {};
  };
  Exports.estimateAttribute = function (obj, attr) {
    let serverData = getServerData(obj);
    let pendingOps = getPendingOps(obj);
    return ObjectStateMutations.estimateAttribute(serverData, pendingOps, obj.className, obj.id, attr);
  };
  Exports.estimateAttributes = function (obj) {
    let serverData = getServerData(obj);
    let pendingOps = getPendingOps(obj);
    return ObjectStateMutations.estimateAttributes(serverData, pendingOps, obj.className, obj.id);
  };
  Exports.commitServerChanges = function (obj, changes) {
    let state = initializeState(obj);
    ObjectStateMutations.commitServerChanges(state.serverData, state.objectCache, changes);
  };
  Exports.enqueueTask = function (obj, task) {
    let state = initializeState(obj);
    return state.tasks.enqueue(task);
  };
  Exports.clearAllState = function () {
    objectState = {};
  };
  Exports.duplicateState = function (source, dest) {
    dest.id = source.id;
  };

  const ObjectStateMutations = include('./ObjectStateMutations', CoreManager);

  let objectState = {};

  function getState(obj) {
    let classData = objectState[obj.className];
    if (classData) {
      return classData[obj.id] || null;
    }
    return null;
  }

  function initializeState(obj, initial) {
    let state = getState(obj);
    if (state) {
      return state;
    }
    if (!objectState[obj.className]) {
      objectState[obj.className] = {};
    }
    if (!initial) {
      initial = ObjectStateMutations.defaultState();
    }
    state = objectState[obj.className][obj.id] = initial;
    return state;
  }

  function getServerData(obj) {
    let state = getState(obj);
    if (state) {
      return state.serverData;
    }
    return {};
  }

  function getPendingOps(obj) {
    let state = getState(obj);
    if (state) {
      return state.pendingOps;
    }
    return [{}];
  }
}