/**
 * Copyright (c) 2015-present, Parse, LLC.
 * All rights reserved.
 *
 * This source code is licensed under the BSD-style license found in the
 * LICENSE file in the root directory of this source tree. An additional grant
 * of patent rights can be found in the PATENTS file in the same directory.
 *
 * 
 */
import include from './include';
export default function (CoreManager, Exports) {

  Exports.default = function (obj) {
    if (!(obj instanceof ParseObject.default)) {
      return true;
    }
    var attributes = obj.attributes;
    for (var attr in attributes) {
      var val = attributes[attr];
      if (!canBeSerializedHelper(val)) {
        return false;
      }
    }
    return true;
  };

  const ParseFile = include('./ParseFile', CoreManager);
  const ParseObject = include('./ParseObject', CoreManager);
  const ParseRelation = include('./ParseRelation', CoreManager);

  function canBeSerializedHelper(value) {
    if (typeof value !== 'object') {
      return true;
    }
    if (value instanceof ParseRelation.default) {
      return true;
    }
    if (value instanceof ParseObject.default) {
      return !!value.id;
    }
    if (value instanceof ParseFile.default) {
      if (value.url()) {
        return true;
      }
      return false;
    }
    if (Array.isArray(value)) {
      for (var i = 0; i < value.length; i++) {
        if (!canBeSerializedHelper(value[i])) {
          return false;
        }
      }
      return true;
    }
    for (var k in value) {
      if (!canBeSerializedHelper(value[k])) {
        return false;
      }
    }
    return true;
  }
}