/**
 * Copyright (c) 2015-present, Parse, LLC.
 * All rights reserved.
 *
 * This source code is licensed under the BSD-style license found in the
 * LICENSE file in the root directory of this source tree. An additional grant
 * of patent rights can be found in the PATENTS file in the same directory.
 *
 * 
 */

import include from './include';
export default function (CoreManager, Exports) {

  Exports.default = function (value, disallowObjects, forcePointers, seen) {
    return encode(value, !!disallowObjects, !!forcePointers, seen || []);
  };

  const ParseACL = include('./ParseACL', CoreManager);
  const ParseFile = include('./ParseFile', CoreManager);
  const ParseGeoPoint = include('./ParseGeoPoint', CoreManager);
  const ParsePolygon = include('./ParsePolygon', CoreManager);
  const ParseObject = include('./ParseObject', CoreManager);
  const ParseOp = include('./ParseOp', CoreManager);
  const ParseRelation = include('./ParseRelation', CoreManager);

  var toString = Object.prototype.toString;

  function encode(value, disallowObjects, forcePointers, seen) {
    if (value instanceof ParseObject.default) {
      if (disallowObjects) {
        throw new Error('Parse Objects not allowed here');
      }
      var seenEntry = value.id ? value.className + ':' + value.id : value;
      if (forcePointers || !seen || seen.indexOf(seenEntry) > -1 || value.dirty() || Object.keys(value._getServerData()).length < 1) {
        return value.toPointer();
      }
      seen = seen.concat(seenEntry);
      return value._toFullJSON(seen);
    }
    if (value instanceof ParseOp.Op || value instanceof ParseACL.default || value instanceof ParseGeoPoint.default || value instanceof ParsePolygon.default || value instanceof ParseRelation.default) {
      return value.toJSON();
    }
    if (value instanceof ParseFile.default) {
      if (!value.url()) {
        throw new Error('Tried to encode an unsaved file.');
      }
      return value.toJSON();
    }
    if (toString.call(value) === '[object Date]') {
      if (isNaN(value)) {
        throw new Error('Tried to encode an invalid date.');
      }
      return { __type: 'Date', iso: value.toJSON() };
    }
    if (toString.call(value) === '[object RegExp]' && typeof value.source === 'string') {
      return value.source;
    }

    if (Array.isArray(value)) {
      return value.map(v => {
        return encode(v, disallowObjects, forcePointers, seen);
      });
    }

    if (value && typeof value === 'object') {
      var output = {};
      for (var k in value) {
        output[k] = encode(value[k], disallowObjects, forcePointers, seen);
      }
      return output;
    }

    return value;
  }
}