/**
 * Copyright (c) 2015-present, Parse, LLC.
 * All rights reserved.
 *
 * This source code is licensed under the BSD-style license found in the
 * LICENSE file in the root directory of this source tree. An additional grant
 * of patent rights can be found in the PATENTS file in the same directory.
 */
import include from './include';
export default function (CoreManager, Exports) {

  Exports.default = equals;

  const ParseACL = include('./ParseACL', CoreManager);
  const ParseFile = include('./ParseFile', CoreManager);
  const ParseGeoPoint = include('./ParseGeoPoint', CoreManager);
  const ParseObject = include('./ParseObject', CoreManager);

  function equals(a, b) {
    if (typeof a !== typeof b) {
      return false;
    }

    if (!a || typeof a !== 'object') {
      // a is a primitive
      return a === b;
    }

    if (Array.isArray(a) || Array.isArray(b)) {
      if (!Array.isArray(a) || !Array.isArray(b)) {
        return false;
      }
      if (a.length !== b.length) {
        return false;
      }
      for (var i = a.length; i--;) {
        if (!equals(a[i], b[i])) {
          return false;
        }
      }
      return true;
    }

    if (a instanceof ParseACL.default || a instanceof ParseFile.default || a instanceof ParseGeoPoint.default || a instanceof ParseObject.default) {
      return a.equals(b);
    }

    if (Object.keys(a).length !== Object.keys(b).length) {
      return false;
    }
    for (var k in a) {
      if (!equals(a[k], b[k])) {
        return false;
      }
    }
    return true;
  }
}