

import include from './include'; /**
                                  * Copyright (c) 2015-present, Parse, LLC.
                                  * All rights reserved.
                                  *
                                  * This source code is licensed under the BSD-style license found in the
                                  * LICENSE file in the root directory of this source tree. An additional grant
                                  * of patent rights can be found in the PATENTS file in the same directory.
                                  *
                                  * 
                                  */

export default function (CoreManager, Exports) {

  Exports.default =

  /**
   * Return an array of unsaved children, which are either Parse Objects or Files.
   * If it encounters any dirty Objects without Ids, it will throw an exception.
   */
  function (obj, allowDeepUnsaved) {
    var encountered = {
      objects: {},
      files: []
    };
    var identifier = obj.className + ':' + obj._getId();
    encountered.objects[identifier] = obj.dirty() ? obj : true;
    var attributes = obj.attributes;
    for (var attr in attributes) {
      if (typeof attributes[attr] === 'object') {
        traverse(attributes[attr], encountered, false, !!allowDeepUnsaved);
      }
    }
    var unsaved = [];
    for (var id in encountered.objects) {
      if (id !== identifier && encountered.objects[id] !== true) {
        unsaved.push(encountered.objects[id]);
      }
    }
    return unsaved.concat(encountered.files);
  };

  const ParseFile = include('./ParseFile', CoreManager);
  const ParseObject = include('./ParseObject', CoreManager);
  const ParseRelation = include('./ParseRelation', CoreManager);

  function traverse(obj, encountered, shouldThrow, allowDeepUnsaved) {
    if (obj instanceof ParseObject.default) {
      if (!obj.id && shouldThrow) {
        throw new Error('Cannot create a pointer to an unsaved Object.');
      }
      var identifier = obj.className + ':' + obj._getId();
      if (!encountered.objects[identifier]) {
        encountered.objects[identifier] = obj.dirty() ? obj : true;
        var attributes = obj.attributes;
        for (var attr in attributes) {
          if (typeof attributes[attr] === 'object') {
            traverse(attributes[attr], encountered, !allowDeepUnsaved, allowDeepUnsaved);
          }
        }
      }
      return;
    }
    if (obj instanceof ParseFile.default) {
      if (!obj.url() && encountered.files.indexOf(obj) < 0) {
        encountered.files.push(obj);
      }
      return;
    }
    if (obj instanceof ParseRelation.default) {
      return;
    }
    if (Array.isArray(obj)) {
      obj.forEach(el => {
        if (typeof el === 'object') {
          traverse(el, encountered, shouldThrow, allowDeepUnsaved);
        }
      });
    }
    for (var k in obj) {
      if (typeof obj[k] === 'object') {
        traverse(obj[k], encountered, shouldThrow, allowDeepUnsaved);
      }
    }
  }
}